using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.NetworkInformation;

namespace AdditionalData
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Web reference to the system
        /// </summary>
        private MSXE173xWeb.MSXE17xx sys;

        /// <summary>
        /// Store the available additional data
        /// </summary>
        private List<int> availableAdditionalData1;
        private List<int> availableAdditionalData2;

        /// <summary>
        /// Store the number of additional data selected
        /// </summary>
        private int selectedAdditionalDataCount = 0;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            // Create the two lists
            this.availableAdditionalData1 = new List<int>();
            this.availableAdditionalData2 = new List<int>();
        }

        /// <summary>
        /// Occurs when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            this.ipTextBox.Text = Properties.Settings.Default.ip;

            // Set to default
            this.channelComboBox.SelectedIndex = 0;
            this.frequencyComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// User clicks on ok
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void okButton_Click(object sender, EventArgs e)
        {
            // Disable the button
            this.okButton.Enabled = false;
            this.ipTextBox.Enabled = false;

            // Start the worker to check the system connection using ping
            systemConnectionBackgroundWorker.RunWorkerAsync();
        }

        /// <summary>
        /// Job of the worker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void systemConnectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            Ping ping = new Ping();
            PingReply reply = ping.Send(this.ipTextBox.Text, 2000);
            if (reply.Status == IPStatus.Success)
            {
                worker.ReportProgress(100);
            }
            else
            {
                worker.ReportProgress(0);
            }
            return;
        }

        /// <summary>
        /// State has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void systemConnectionBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (e.ProgressPercentage == 100)
            {
                // success

                // Create the web reference
                this.sys = new MSXE173xWeb.MSXE17xx();
                this.sys.Url = "http://" + this.ipTextBox.Text + ":5555";
                this.sys.Timeout = 2000;

                // update status
                this.currentStatusLabel.Text = "Connected to " + this.ipTextBox.Text;
                this.currentStatusLabel.ForeColor = Color.Green;
                this.channelSelectionGroupBox.Enabled = true;
            }
            else
            {
                // connection problem
                this.currentStatusLabel.Text = "Cannot open connection to " + this.ipTextBox.Text;
                this.currentStatusLabel.ForeColor = Color.Red;
                this.ipTextBox.Enabled = true;
                this.okButton.Enabled = true;
            }
        }

        /// <summary>
        /// Click on get available additional data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getAvailableADButton_Click(object sender, EventArgs e)
        {
            ulong idNumberLsb = 0;
            ulong idNumberMsb = 0;
            ulong serialNumberLsb = 0;
            ulong serialNumberMsb = 0;
            ulong model = 0;
            ulong mode = 0;
            ulong positionSize = 0;
            ulong signalPeriod = 0;
            ulong stepPerRevolution = 0;
            ulong numberOfRevolution = 0;
            ulong scalingFactor = 0;
            ulong additionalData = 0;
            ulong additionalData1 = 0;
            ulong additionalData2 = 0;

            // Clear items
            this.additionalData1ListView.Items.Clear();
            this.additionalData2ListView.Items.Clear();

            // Clear lists
            this.availableAdditionalData1.Clear();
            this.availableAdditionalData2.Clear();

            this.currentStatusLabel.Text = "";
            this.currentStatusLabel.Invalidate();

            // Get properties
            try
            {
                MSXE173xWeb.DefaultResponse response = this.sys.MFEndatGetSensorProperties((ulong)channelComboBox.SelectedIndex,
                    (ulong)0,
                    0, 0, 0, 0,
                    out idNumberLsb,
                    out idNumberMsb,
                    out serialNumberLsb,
                    out serialNumberMsb,
                    out model,
                    out mode,
                    out positionSize,
                    out signalPeriod,
                    out stepPerRevolution,
                    out numberOfRevolution,
                    out scalingFactor,
                    out additionalData);
                if (response.iReturnValue != 0)
                {
                    this.currentStatusLabel.Text = "MFEndatGetSensorProperties error : " + response.iReturnValue.ToString() + ". syserrno : " + response.syserrno.ToString();
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }
            catch (Exception ex)
            {
                this.currentStatusLabel.Text = "MFEndatGetSensorProperties error : " + ex.Message;
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            // Success
            this.currentStatusLabel.Text = "MFEndatGetSensorProperties ok";
            this.currentStatusLabel.ForeColor = Color.Green;

            // Check if the sensor is compatible with EnDat 2.2
            if (mode != 1)
            {
                this.currentStatusLabel.Text = "The selected sensor is not compatible with EnDat 2.2. The additional data are not available";
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            // Now get the available additional data
            additionalData1 = additionalData & 0xFFFF;
            additionalData2 = (additionalData >> 16) & 0xFFFF;

            // Use page 85/121 and page 34/121 of EnDat specifications 
            // Additional data 1
            // Always
            this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Send additional info 1 without data contents", "0x40" }));
            this.availableAdditionalData1.Add(0x40);
            if ((additionalData1 & 1) == 1)
            {
                this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Position value 2 word 1 LSB", "0x42" }));
                this.availableAdditionalData1.Add(0x42);
                this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Position value 2 word 2", "0x43" }));
                this.availableAdditionalData1.Add(0x43);
                this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Position value 2 word 3 MSB", "0x44" }));
                this.availableAdditionalData1.Add(0x44);
            }
            if (((additionalData1 >> 1) & 1) == 1)
            {
                this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Test values word 1 LSB", "0x49" }));
                this.availableAdditionalData1.Add(0x49);
                this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Test values word 2", "0x4A" }));
                this.availableAdditionalData1.Add(0x4A);
                this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Test values word 3 MSB", "0x4B" }));
                this.availableAdditionalData1.Add(0x4B);
            }
            if (((additionalData1 >> 2) & 1) == 1)
            {
                this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Temperature sensor 1 (external)", "0x4C" }));
                this.availableAdditionalData1.Add(0x4C);
            }
            if (((additionalData1 >> 3) & 1) == 1)
            {
                this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Temperature sensor 2 (external)", "0x4D" }));
                this.availableAdditionalData1.Add(0x4D);
            }
            if (((additionalData1 >> 4) & 1) == 1)
            {
                this.additionalData1ListView.Items.Add(new ListViewItem(new string[2] { "Additional sensors", "0x4E" }));
                this.availableAdditionalData1.Add(0x4E);
            }
            // Additional data 2
            // Always
            this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Send additional info 2 without data contents", "0x50" }));
            this.availableAdditionalData2.Add(0x50);
            if ((additionalData2 & 1) == 1)
            {
                this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Commutation", "0x51" }));
                this.availableAdditionalData2.Add(0x51);
            }
            if (((additionalData2 >> 1) & 1) == 1)
            {
                this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Acceleration", "0x52" }));
                this.availableAdditionalData2.Add(0x52);
            }
            if (((additionalData2 >> 2) & 1) == 1)
            {
                this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Limit position signals", "0x54" }));
                this.availableAdditionalData2.Add(0x54);
            }
            if (((additionalData2 >> 3) & 1) == 1)
            {
                this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Asynchronous position value word 1 LSB", "0x56" }));
                this.availableAdditionalData2.Add(0x56);
                this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Asynchronous position value word 2", "0x57" }));
                this.availableAdditionalData2.Add(0x57);
                this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Asynchronous position value word 3 MSB", "0x58" }));
                this.availableAdditionalData2.Add(0x58);
            }
            if (((additionalData2 >> 4) & 1) == 1)
            {
                this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Operating status error sources", "0x59" }));
                this.availableAdditionalData2.Add(0x59);
            }
            if (((additionalData2 >> 5) & 1) == 1)
            {
                // Normally reserved but not assigned for the moment (no MRS code)
                // this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Reserved", "0" }));
            }
            if (((additionalData2 >> 6) & 1) == 1)
            {
                // Normally position value 2 but not assigned for the moment (no MRS code)
                // this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Position value 2", "0" }));
            }
            if (((additionalData2 >> 7) & 1) == 1)
            {
                this.additionalData2ListView.Items.Add(new ListViewItem(new string[2] { "Timestamp", "0x5B" }));
                this.availableAdditionalData2.Add(0x5B);
            }

            // Update
            this.additionalData1ListView.Refresh();
            this.additionalData2ListView.Refresh();
        }

        /// <summary>
        /// Click on the initialise button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initButton_Click(object sender, EventArgs e)
        {
            ulong frequency = 0;

            this.currentStatusLabel.Text = "";
            this.currentStatusLabel.Invalidate();

            // Get the frequency
            switch (this.frequencyComboBox.SelectedIndex)
            {
                case 0:
                    frequency = 500;
                    break;
                case 1:
                    frequency = 900;
                    break;
                case 2:
                    frequency = 1500;
                    break;
                case 3:
                    frequency = 2500;
                    break;
                case 4:
                    frequency = 4500;
                    break;
                default:
                    return;
            }

            // Initialise the sensor
            try
            {
                int syserrno = 0;
                int retval = this.sys.MFEndatInitSensor((ulong)this.channelComboBox.SelectedIndex,
                    (ulong)0,
                    frequency,
                    0, 0, 0, 0,
                    out syserrno);
                if (retval != 0)
                {
                    this.currentStatusLabel.Text = "MFEndatInitSensor error : " + retval.ToString() + ". syserrno : " + syserrno.ToString();
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }
            catch (Exception ex)
            {
                this.currentStatusLabel.Text = "MFEndatInitSensor error : " + ex.Message;
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            // Success
            this.currentStatusLabel.Text = "MFEndatInitSensor ok";
            this.currentStatusLabel.ForeColor = Color.Green;
        }

        /// <summary>
        /// Occurs when the text is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void adCounttextBox_TextChanged(object sender, EventArgs e)
        {
            int count = 0;
            try
            {
                count = Convert.ToInt32(this.adCounttextBox.Text);
                if (count > 2)
                    count = 0;
            }
            catch (Exception)
            {
                count = 0;
            }

            switch (count)
            {
                case 0:
                    this.mrsCodeTextBox1.Enabled = false;
                    this.mrsCodeTextBox2.Enabled = false;
                    break;
                case 1:
                    this.mrsCodeTextBox1.Enabled = true;
                    this.mrsCodeTextBox2.Enabled = false;
                    break;
                case 2:
                    this.mrsCodeTextBox1.Enabled = true;
                    this.mrsCodeTextBox2.Enabled = true;
                    break;
            }
        }

        /// <summary>
        /// Get value (hexa or decimal)
        /// </summary>
        /// <param name="text"></param>
        /// <param name="correct"></param>
        /// <returns></returns>
        private ulong GetValue(String text, ref bool correct)
        {
            ulong value = 0;
            correct = false;

            if (text.Length == 0)
                return 0;

            // Hexa ? 
            if ((text.Length > 2) && (text[0] == '0') && (text[1] == 'x'))
            {
                try
                {
                    String tmp = text.Substring(2);
                    value = Convert.ToUInt64(tmp, 16);
                    correct = true;
                    return value;
                }
                catch (Exception)
                {
                    return 0;
                }
            }

            // Decimal
            try
            {
                value = Convert.ToUInt64(text);
                correct = true;
                return value;
            }
            catch (Exception)
            {
                return 0;
            }
        }

        /// <summary>
        /// Click on select additional data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void selectAdditionalDataButton_Click(object sender, EventArgs e)
        {
            ulong aiCount = 0;
            ulong mrsCode1 = 0;
            ulong mrsCode2 = 0;
            bool correct = false;

            // Get the additional data count
            try
            {
                aiCount = Convert.ToUInt64(this.adCounttextBox.Text);
            }
            catch (Exception)
            {
                this.currentStatusLabel.Text = "Invalid number of additional data " + this.adCounttextBox.Text;
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            if (aiCount > 0)
            {
                // Get the mrs code for additional data 1
                mrsCode1 = GetValue(this.mrsCodeTextBox1.Text, ref correct);
                if (correct == false)
                {
                    this.currentStatusLabel.Text = "Invalid MRS code for additional data 1 " + this.mrsCodeTextBox1.Text;
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
                // Is in the list
                if (this.availableAdditionalData1.Contains((int)mrsCode1) == false)
                {
                    this.currentStatusLabel.Text = "Invalid MRS code for additional data 1 (not in the list) " + this.mrsCodeTextBox1.Text;
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }
            if (aiCount > 1)
            {
                // Get the mrs code for additional data 2
                mrsCode2 = GetValue(this.mrsCodeTextBox2.Text, ref correct);
                if (correct == false)
                {
                    this.currentStatusLabel.Text = "Invalid MRS code for additional data 2 " + this.mrsCodeTextBox2.Text;
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
                // Is in the list
                if (this.availableAdditionalData2.Contains((int)mrsCode2) == false)
                {
                    this.currentStatusLabel.Text = "Invalid MRS code for additional data 2 (not in the list) " + this.mrsCodeTextBox1.Text;
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }

            // Select the additional datas
            try
            {
                int syserrno = 0;
                int retval = this.sys.MFEndatSelectAdditionalData((ulong)this.channelComboBox.SelectedIndex,
                    0,
                    aiCount,
                    mrsCode1,
                    mrsCode2,
                    0, 0, 0, 0,
                    out syserrno);
                if (retval != 0)
                {
                    this.currentStatusLabel.Text = "MFEndatSelectAdditionalData error : " + retval.ToString() + ". syserrno : " + syserrno.ToString();
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }
            catch (Exception ex)
            {
                this.currentStatusLabel.Text = "MFEndatSelectAdditionalData error : " + ex.Message;
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            // Success
            this.currentStatusLabel.Text = "MFEndatSelectAdditionalData ok";
            this.currentStatusLabel.ForeColor = Color.Green;

            // Store the number of additional data selected
            this.selectedAdditionalDataCount = (int)aiCount;
        }

        /// <summary>
        /// Click on get position with additional data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getPositionButton_Click(object sender, EventArgs e)
        {
            ulong positionlow = 0;
            ulong positionhigh = 0;
            ulong addinfo1 = 0;
            ulong addinfo2 = 0;

            // Clear
            this.valuesListView.Items.Clear();

            this.currentStatusLabel.Text = "";
            this.currentStatusLabel.Invalidate();

            // Get values
            try
            {
                MSXE173xWeb.DefaultResponse response =  this.sys.MFEndatGetPositionWithAddData((ulong)channelComboBox.SelectedIndex,
                    (ulong)0,
                    0, 0, 0, 0,
                    out positionlow,
                    out positionhigh,
                    out addinfo1,
                    out addinfo2);
                if (response.iReturnValue != 0)
                {
                    this.currentStatusLabel.Text = "MFEndatGetPositionWithAddData error : " + response.iReturnValue.ToString() + ". syserrno : " + response.syserrno.ToString();
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }
            catch (Exception ex)
            {
                this.currentStatusLabel.Text = "MFEndatGetPositionWithAddData error : " + ex.Message;
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            // Success
            this.currentStatusLabel.Text = "MFEndatGetPositionWithAddData ok";
            this.currentStatusLabel.ForeColor = Color.Green;

            // Add the values
            this.valuesListView.Items.Add(new ListViewItem(new string[2] { "Position low", positionlow.ToString() }));
            this.valuesListView.Items.Add(new ListViewItem(new string[2] { "Position high", positionhigh.ToString() }));
            if (this.selectedAdditionalDataCount >= 1)
            {
                this.valuesListView.Items.Add(new ListViewItem(new string[2] { "Additional data 1", addinfo1.ToString() }));
            }
            if (this.selectedAdditionalDataCount == 2)
            {
                this.valuesListView.Items.Add(new ListViewItem(new string[2] { "Additional data 2", addinfo2.ToString() }));
            }
        }
    }
}