using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.ComponentModel;

namespace EnDatLatchAcquisition
{
    public class DataClient
    {
        /// <summary>
        /// Connection with the system
        /// </summary>
        private Socket socket;

        /// <summary>
        /// The ip of the system
        /// </summary>
        private String systemIp;

        /// <summary>
        /// The port that we listen to
        /// </summary>
        private int systemPort;

        /// <summary>
        /// The size of one packet
        /// </summary>
        private int [] size;

        /// <summary>
        /// Background worker that enables to handle the socket reception
        /// </summary>
        private BackgroundWorker worker;

        /// <summary>
        /// Delegate to become event when a packet is received
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public delegate void PacketReceivedEventHandler(object sender, DataClientEventArgs e);

        /// <summary>
        /// Definition of the evenement
        /// </summary>
        public event PacketReceivedEventHandler packetReceived;

        public int receptedCount = 0;

        /// <summary>
        /// Constructor of the class
        /// </summary>
        /// <param name="ipAddress">The ip address of the system</param>
        /// <param name="port">The port used</param>
        /// <param name="packetSize">Array that contains the size per packet</param>
        public DataClient(String ipAddress, int port, int [] packetSize)
        {
            socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            // Copy params
            this.systemIp = ipAddress;
            this.systemPort = port;
            this.size = packetSize;

            // Create the background worker
            worker = new BackgroundWorker();
            worker.WorkerSupportsCancellation = true;
            worker.WorkerReportsProgress = true;
            worker.DoWork += new DoWorkEventHandler(worker_DoWork);
        }

        /// <summary>
        /// Return the maximum size
        /// </summary>
        /// <returns></returns>
        private int getMaximumSize()
        {
            int max = 0;
            for (int i = 0; i < this.size.Length; i++)
            {
                if (this.size[i] > max)
                    max = this.size[i];
            }
            return max;
        }

        /// <summary>
        /// Work of the background worker
        /// Receive data from the socket
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            // Allocate space
            byte[] buffer = new byte[32768];

            DataClientEventArgs eventArgs = new DataClientEventArgs();
            eventArgs.data = new byte[this.getMaximumSize()];
            int eventBufferFreeSize = 0;
            int eventBufferIndex = 0;
            bool indexKnown = false;

            try
            {
                while (this.worker.CancellationPending == false)
                {
                    // Receive a large chunk
                    // Receiving a large chunk to then convert it is faster than receiving small chunk of data.
                    int recepted = socket.Receive(buffer);

                    receptedCount += recepted;

                    int offset = 0;

                    while (recepted > 0)
                    {
                        if (indexKnown == false)
                        {
                            uint eventsrc = BitConverter.ToUInt32(buffer, offset);
                            int chanindex = (int)(eventsrc >> 16);
                            if (chanindex >= this.size.Length)
                            {
                                Console.WriteLine("Wrong channel index.");
                                return;
                            }
                            eventArgs.channel = chanindex;
                            eventBufferFreeSize = this.size[chanindex];
                            indexKnown = true;
                        }

                        int chunk = (recepted > eventBufferFreeSize) ? eventBufferFreeSize : recepted;
                        // Copy buffer
                        Buffer.BlockCopy(buffer, offset, eventArgs.data, eventBufferIndex, chunk);
                        // Update
                        recepted -= chunk;
                        offset += chunk;
                        eventBufferIndex += chunk;
                        eventBufferFreeSize -= chunk;
                        // Do we have to send ?
                        if (eventBufferFreeSize == 0)
                        {
                            OnPacketReceived(eventArgs);
                            indexKnown = false;
                            // Reset
                            eventBufferFreeSize = 0;
                            eventBufferIndex = 0;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error : " + ex.Message);
            }
        }

        /// <summary>
        /// Signal the event to the listeners
        /// </summary>
        /// <param name="e"></param>
        protected virtual void OnPacketReceived(DataClientEventArgs e)
        {
            if (packetReceived != null)
            {
                packetReceived(this, e);
            }
        }

        /// <summary>
        /// Start to listen from the packet
        /// </summary>
        /// <returns></returns>
        public int StartListening()
        {
            // Try to connect
            try
            {
                socket.Connect(systemIp, systemPort);
            }
            catch (Exception ex)
            {
                Console.WriteLine("socket connection exception" + ex.Message);
                return -1;
            }

            worker.RunWorkerAsync();
            return 0;
        }

        /// <summary>
        /// Close connection
        /// </summary>
        public void Close()
        {
            // Close the connection
            try
            {
                socket.Close();
            }
            catch (Exception) { };

            // Stop the worker
            worker.CancelAsync();
            worker.Dispose();
        }

        /// <summary>
        /// Print available bytes in the socket
        /// </summary>
        public void PrintAvailable()
        {
            try
            {
                Console.WriteLine("Available : " + socket.Available);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception : " + ex.Message);
            }
        }
    }
}
