using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace EnDatLatchAcquisition
{
    public partial class EndatChannel : UserControl
    {
        private int index = 0;
        private ulong model = 0;
        private ulong dataformat = 0;
        private int packetCount = 0;
        private object viewLock = new object();
        private byte[] data = null;
        private int packetsize = 0;
        private bool dataAvailable = false;

        public EndatChannel()
        {
            InitializeComponent();

            // Start the timer
            this.displayTimer.Start();
        }

        [Category("Configuration"), Browsable(true), Description("Index of the channel")]
        public int Index
        {
            get
            {
                return this.index;
            }
            set
            {
                this.index = value;
                this.endatGroupBox.Text = "EnDat channel " + this.index.ToString();
            }
        }

        /// <summary>
        /// Get/Set the value of the model
        /// The model represents the type of the EnDat sensor
        /// </summary>
        public ulong Model
        {
            get
            {
                return this.model;
            }
            set
            {
                this.model = value;
                this.updateValueListView();
            }
        }

        /// <summary>
        /// Get/Set the value of the data format
        /// </summary>
        public ulong DataFormat
        {
            get
            {
                return this.dataformat;
            }
            set
            {
                this.dataformat = value;
                this.updateValueListView();
            }
        }

        /// <summary>
        /// Set/Get the size of a packet in byte
        /// </summary>
        public int PacketSize
        {
            get
            {
                return this.packetsize;
            }
            set
            {
                this.packetsize = value;
                this.data = new byte[this.packetsize];
            }
        }

        /// <summary>
        /// Update the value list view
        /// </summary>
        private void updateValueListView()
        {
            this.dataAvailable = false;
            this.packetCount = 0;

            lock (this.viewLock)
            {
                this.valuesListView.Items.Clear();

                // Event source is always there
                this.valuesListView.Items.Add(new ListViewItem(new string[] { "Event source", "0" }));

                // Then we have the data
                if ((this.dataformat & 16) != 16)
                {
                    // default
                    this.valuesListView.Items.Add(new ListViewItem(new string[] { "Position (LSB)", "0" }));
                    this.valuesListView.Items.Add(new ListViewItem(new string[] { "Position (MSB)", "0" }));
                }
                // Standardised
                else
                {
                    // absolute linear
                    if ((this.model == 4) || (this.model == 6))
                    {
                        this.valuesListView.Items.Add(new ListViewItem(new string[] { "Position (m)", "0" }));
                    }
                    // multi-turn
                    else if ((this.model == 13) || (this.model == 14))
                    {
                        this.valuesListView.Items.Add(new ListViewItem(new string[] { "Position (LSB)", "0" }));
                        this.valuesListView.Items.Add(new ListViewItem(new string[] { "Position (MSB)", "0" }));
                        this.valuesListView.Items.Add(new ListViewItem(new string[] { "Angle ()", "0" }));
                    }
                    else
                    {
                        // default
                        this.valuesListView.Items.Add(new ListViewItem(new string[] { "Position (LSB)", "0" }));
                        this.valuesListView.Items.Add(new ListViewItem(new string[] { "Position (MSB)", "0" }));
                    }
                }

                // Then the error sources
                this.valuesListView.Items.Add(new ListViewItem(new string[] { "Errors", "0" }));

                // Time stamp
                if ((this.dataformat & 1) == 1)
                {
                    this.valuesListView.Items.Add(new ListViewItem(new string[] { "Time stamp (s)", "0" }));
                    this.valuesListView.Items.Add(new ListViewItem(new string[] { "Time stamp (s)", "0" }));
                }

                // Digital I/Os
                if ((this.dataformat & 2) == 2)
                {
                    this.valuesListView.Items.Add(new ListViewItem(new string[] { "Digital I/O", "0" }));
                }

                // Additional data 1
                if ((this.dataformat & 4) == 4)
                {
                    this.valuesListView.Items.Add(new ListViewItem(new string[] { "Additional data 1", "0" }));
                }

                // Additional data 2
                if ((this.dataformat & 8) == 8)
                {
                    this.valuesListView.Items.Add(new ListViewItem(new string[] { "Additional data 2", "0" }));
                }

                this.valuesListView.Update();
            }
        }

        /// <summary>
        /// New value are available
        /// </summary>
        /// <param name="data"></param>
        public void fireNewValue(byte[] data)
        {
            this.packetCount++;

            // copy
            for (int i = 0; (i < data.Length) && (i < this.packetsize); i++)
            {
                this.data[i] = data[i];
            }

            this.dataAvailable = true;
        }

        /// <summary>
        /// Occurs when the display timer ticks
        /// Update the value of the list view
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void displayTimer_Tick(object sender, EventArgs e)
        {
            if (this.dataAvailable == true)
            {
                lock (this.viewLock)
                {
                    int index = 0;
                    int indexbuffer = 0;

                    // event source
                    this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer).ToString();
                    index += 1;
                    indexbuffer += 4;

                    // Then we have the data
                    if ((this.dataformat & 16) != 16)
                    {
                        // default - two 32 bits integer
                        this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer).ToString();
                        this.valuesListView.Items[index+1].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer+4).ToString();
                        index += 2;
                        indexbuffer += 8;
                    }
                    // Standardised
                    else
                    {
                        // absolute linear
                        if ((this.model == 4) || (this.model == 6))
                        {
                            this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToSingle(this.data, indexbuffer).ToString();
                            index += 1;
                            indexbuffer += 4;
                        }
                        // multi-turn
                        else if ((this.model == 13) || (this.model == 14))
                        {
                            this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer).ToString();
                            this.valuesListView.Items[index+1].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer+4).ToString();
                            this.valuesListView.Items[index+2].SubItems[1].Text = ((double)BitConverter.ToSingle(this.data, indexbuffer+8)).ToString();
                            index += 3;
                            indexbuffer += 12;
                        }
                        else
                        {
                            // default
                            this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer).ToString();
                            this.valuesListView.Items[index+1].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer+4).ToString();
                            index += 2;
                            indexbuffer += 8;
                        }
                    }

                    // Then the error sources
                    this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer).ToString();
                    index += 1;
                    indexbuffer += 4;

                    // Time stamp
                    if ((this.dataformat & 1) == 1)
                    {
                        this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer).ToString();
                        this.valuesListView.Items[index+1].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer+4).ToString();
                        index += 2;
                        indexbuffer += 8;
                    }

                    // Digital I/Os
                    if ((this.dataformat & 2) == 2)
                    {
                        this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer).ToString();
                        index += 1;
                        indexbuffer += 4;
                    }

                    // Additional data 1
                    if ((this.dataformat & 4) == 4)
                    {
                        this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer).ToString();
                        index += 1;
                        indexbuffer += 4;
                    }

                    // Additional data 2
                    if ((this.dataformat & 8) == 8)
                    {
                        this.valuesListView.Items[index].SubItems[1].Text = BitConverter.ToUInt32(this.data, indexbuffer).ToString();
                        index += 1;
                        indexbuffer += 4;
                    }
                }
            }
            this.packetCountLabel.Text = this.packetCount.ToString();
        }
    }
}
