using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace EnDatLatchAcquisition
{
    public partial class MainForm : Form
    {
        private MSXE173x sys;
        private DataClient client;

        /// <summary>
        /// Useful graphics variables
        /// </summary>
        private CheckBox[] channelCheckboxes;
        private EndatChannel[] endatChannels;

        public MainForm()
        {
            InitializeComponent();

            // Create the useful graphics variables
            this.channelCheckboxes = new CheckBox[4];
            this.channelCheckboxes[0] = this.channelCheckBox0;
            this.channelCheckboxes[1] = this.channelCheckBox1;
            this.channelCheckboxes[2] = this.channelCheckBox2;
            this.channelCheckboxes[3] = this.channelCheckBox3;
            this.endatChannels = new EndatChannel[4];
            this.endatChannels[0] = this.endatChannel0;
            this.endatChannels[1] = this.endatChannel1;
            this.endatChannels[2] = this.endatChannel2;
            this.endatChannels[3] = this.endatChannel3;
        }

        /// <summary>
        /// Occurs when the form is loaded for the first time
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            this.ipTextBox.Text = Properties.Settings.Default.ip;

            // Default
            this.frequencyComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// User clicks on the initialise button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initButton_Click(object sender, EventArgs e)
        {
            ulong frequency = 0;
            int selectedcount = 0;

            this.latchGroupBox.Enabled = false;

            switch (frequencyComboBox.SelectedIndex)
            {
                case 0:
                    frequency = 500;
                    break;
                case 1:
                    frequency = 900;
                    break;
                case 2:
                    frequency = 1500;
                    break;
                case 3:
                    frequency = 2500;
                    break;
                case 4:
                    frequency = 4500;
                    break;
            }

            // Initialise the selected channels
            for (int i = 0; i < this.channelCheckboxes.Length; i++)
            {
                if (this.channelCheckboxes[i].Checked == true)
                {
                    int retval = 0;
                    string error = String.Empty;
                    retval = this.sys.EndatInitSensor((ulong)i, frequency, out error);
                    if (retval != 0)
                    {
                        this.currentStatusLabel.Text = "Cannot initialise sensor " + i.ToString() + ": " + error;
                        this.currentStatusLabel.ForeColor = Color.Red;
                        return;
                    }
                    selectedcount += 1;
                }
            }

            if (selectedcount == 0)
                return;

            // Disable
            for (int i = 0; i < this.channelCheckboxes.Length; i++)
            {
                this.channelCheckboxes[i].Enabled = false;
            }

            // Success
            this.currentStatusLabel.Text = "Selected sensors are now initialised.";
            this.currentStatusLabel.ForeColor = Color.Black;
            this.latchGroupBox.Enabled = true;
            this.initButton.Enabled = false;
        }

        /// <summary>
        /// Click on ok
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void okButton_Click(object sender, EventArgs e)
        {
            // Try to create the system
            try
            {
                this.sys = new MSXE173x();
                this.sys.Ip = this.ipTextBox.Text;
            }
            catch (Exception ex)
            {
                this.currentStatusLabel.Text = "Cannot open connection to the system: " + ex.Message;
                return;
            }

            // Allow some controls
            this.okButton.Enabled = false;
            this.initGroupBox.Enabled = true;
        }

        /// <summary>
        /// Construct the source parameter
        /// </summary>
        /// <param name="source"></param>
        /// <param name="edgecount"></param>
        /// <returns></returns>
        private int GetLatchSource(out ulong source, out ulong edgecount)
        {
            source = 0;
            edgecount = 0;

            if (synchroTriggerCheckBox.Checked == true)
                source += 2;

            if (hwTriggerRisingCheckBox.Checked == true)
                source += 4;

            if (hwTriggerFallingcheckBox.Checked == true)
                source += 8;

            if ((hwTriggerRisingCheckBox.Checked == true) || (hwTriggerFallingcheckBox.Checked == true))
                source += 1;

            // Check the source
            if (source == 0)
            {
                this.currentStatusLabel.Text = "Please select at least the synchro trigger or the hardware trigger.";
                this.currentStatusLabel.ForeColor = Color.Red;
                return -1;
            }

            // Have to get the edge count
            if ((hwTriggerRisingCheckBox.Checked == true) || (hwTriggerFallingcheckBox.Checked == true))
            {
                try
                {
                    edgecount = Convert.ToUInt64(this.edgeCountTextBox.Text);
                    if (edgecount <= 0)
                    {
                        this.currentStatusLabel.Text = "Invalid edge count. Cannot be lower or equal to 0.";
                        return -1;
                    }
                }
                catch (Exception ex)
                {
                    this.currentStatusLabel.Text = "Invalid edge count : " + ex.Message;
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return -1;
                }
            }

            return 0;
        }

        /// <summary>
        /// Construct the dataformat parameter
        /// </summary>
        /// <param name="dataformat"></param>
        private void GetDataFormat(out ulong dataformat)
        {
            dataformat = 0;

            if (timestampCheckBox.Checked == true)
                dataformat += 1;

            if (digioCheckBox.Checked == true)
                dataformat += 2;

            if (ad1CheckBox.Checked == true)
                dataformat += 4;

            if (ad2CheckBox.Checked == true)
                dataformat += 8;

            if (standardisedCheckBox.Checked == true)
                dataformat += 16;
        }

        /// <summary>
        /// User clicks on start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            ulong source = 0;
            ulong edgecount = 0;
            ulong dataformat = 0;
            int[] sizes = new int[4];
            int retval = 0;
            string error = string.Empty;

            // Construct the parameters
            if (GetLatchSource(out source, out edgecount) != 0)
                return;

            GetDataFormat(out dataformat);

            // Set the information on the EnDat channels
            for (int i = 0; i < this.endatChannels.Length; i++)
            {
                if (this.channelCheckboxes[i].Checked == true)
                {
                    ulong model = 0;
                    retval = this.sys.EndatGetSensorModel((ulong)i, out model, out error);

                    if (retval != 0)
                    {
                        this.currentStatusLabel.Text = "Cannot get the model for sensor " + i.ToString() + ": " + error;
                        this.currentStatusLabel.ForeColor = Color.Red;
                        return;
                    }

                    // Set
                    this.endatChannels[i].Model = model;
                    this.endatChannels[i].DataFormat = dataformat;
                }
            }

            // Get for each channel the size that is send by the dataserver
            for (int i = 0; i < this.channelCheckboxes.Length; i++)
            {
                if (this.channelCheckboxes[i].Checked == true)
                {
                    int sz = 0;
                    retval = this.sys.EndatGetPacketSize(dataformat, (ulong)i, out sz, out error);
                    if (retval != 0)
                    {
                        this.currentStatusLabel.Text = "Cannot get the size of the packet for sensor " + i.ToString() + ": " + error;
                        this.currentStatusLabel.ForeColor = Color.Red;
                        return;
                    }
                    sizes[i] = sz;
                    this.endatChannels[i].PacketSize = sz;
                }
                else
                {
                    sizes[i] = 0;
                }
            }

            // Start the logic
            for (int i = 0; i < this.channelCheckboxes.Length; i++)
            {
                if (this.channelCheckboxes[i].Checked == true)
                {
                    retval = this.sys.EndatInitAndEnableLatchPositionValues((ulong)i, source, edgecount, dataformat, out error);
                    if (retval != 0)
                    {
                        this.currentStatusLabel.Text = "Cannot start latch logic on sensor " + i.ToString() + ": " + error;
                        this.currentStatusLabel.ForeColor = Color.Red;

                        // Disable the started one
                        for (int j = 0; j < i; j++)
                        {
                            this.sys.EndatDisableAndReleaseLatchPositionValues((ulong)j, out error);
                        }

                        return;
                    }
                }
            }

            // create the data client
            this.client = new DataClient(this.ipTextBox.Text, 8989, sizes);

            // add listener
            this.client.packetReceived += new DataClient.PacketReceivedEventHandler(client_packetReceived);

            // start
            retval = this.client.StartListening();
            if (retval != 0)
            {
                this.currentStatusLabel.Text = "Cannot open a connection to the dataserver or the system.";
                this.currentStatusLabel.ForeColor = Color.Red;
                for (int i = 0; i < this.channelCheckboxes.Length; i++)
                {
                    if (this.channelCheckboxes[i].Checked == true)
                    {
                        this.sys.EndatDisableAndReleaseLatchPositionValues((ulong)i, out error);
                    }
                }
                return;
            }

            // Success
            this.currentStatusLabel.Text = "Latch acquisition started.";
            this.currentStatusLabel.ForeColor = Color.Black;

            // Disable the start button
            this.startButton.Enabled = false;

            EnableDisableLatchConfiguration(false);
        }

        /// <summary>
        /// Get a packet
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void client_packetReceived(object sender, DataClientEventArgs e)
        {
            if (e.channel >= this.endatChannels.Length)
            {
                Console.WriteLine("Invalid channel: " + e.channel.ToString());
                return;
            }

            // Fire the new values
            this.endatChannels[e.channel].fireNewValue(e.data);
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < this.channelCheckboxes.Length; i++)
            {
                if (this.channelCheckboxes[i].Checked == true)
                {
                    string error = string.Empty;
                    this.sys.EndatDisableAndReleaseLatchPositionValues((ulong)i, out error);
                }
            }

            if (this.client != null)
            {
                this.client.Close();
                this.client = null;
            }

            // Success
            this.currentStatusLabel.Text = "Latch acquisition stopped.";
            this.currentStatusLabel.ForeColor = Color.Black;

            // Enable the start button
            this.startButton.Enabled = true;

            EnableDisableLatchConfiguration(true);
        }

        /// <summary>
        /// Enable/disable the latch configuration
        /// </summary>
        /// <param name="state"></param>
        private void EnableDisableLatchConfiguration(bool state)
        {
            this.timestampCheckBox.Enabled = state;
            this.digioCheckBox.Enabled = state;
            this.ad1CheckBox.Enabled = state;
            this.ad2CheckBox.Enabled = state;
            this.standardisedCheckBox.Enabled = state;
            this.synchroTriggerCheckBox.Enabled = state;
            this.hwTriggerFallingcheckBox.Enabled = state;
            this.hwTriggerRisingCheckBox.Enabled = state;
            this.edgeCountTextBox.Enabled = state;
        }

        private void MainForm_Resize(object sender, EventArgs e)
        {
            
        }
    }
}