using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.NetworkInformation;

namespace GetPosition
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Web reference to the system
        /// </summary>
        private MSXE173xWeb.MSXE17xx sys;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Occurs when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            this.ipTextBox.Text = Properties.Settings.Default.ip;

            // Set to default
            this.channelComboBox.SelectedIndex = 0;
            this.frequencyComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Job of the worker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void systemConnectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            Ping ping = new Ping();
            PingReply reply = ping.Send(this.ipTextBox.Text, 2000);
            if (reply.Status == IPStatus.Success)
            {
                worker.ReportProgress(100);
            }
            else
            {
                worker.ReportProgress(0);
            }
            return;
        }

        /// <summary>
        /// State has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void systemConnectionBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (e.ProgressPercentage == 100)
            {
                // success

                // Create the web reference
                this.sys = new MSXE173xWeb.MSXE17xx();
                this.sys.Url = "http://" + this.ipTextBox.Text + ":5555";
                this.sys.Timeout = 2000;

                // update status
                this.currentStatusLabel.Text = "Connected to " + this.ipTextBox.Text;
                this.currentStatusLabel.ForeColor = Color.Green;
                this.channelSelectionGroupBox.Enabled = true;
            }
            else
            {
                // connection problem
                this.currentStatusLabel.Text = "Cannot open connection to " + this.ipTextBox.Text;
                this.currentStatusLabel.ForeColor = Color.Red;
                this.ipTextBox.Enabled = true;
                this.okButton.Enabled = true;
            }
        }

        /// <summary>
        /// User clicks on ok
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void okButton_Click(object sender, EventArgs e)
        {
            // Disable the button
            this.okButton.Enabled = false;
            this.ipTextBox.Enabled = false;

            // Start the worker to check the system connection using ping
            systemConnectionBackgroundWorker.RunWorkerAsync();
        }

        /// <summary>
        /// Click on the initialise button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initButton_Click(object sender, EventArgs e)
        {
            ulong frequency = 0;

            this.currentStatusLabel.Text = "";
            this.currentStatusLabel.Invalidate();

            // Get the frequency
            switch (this.frequencyComboBox.SelectedIndex)
            {
                case 0:
                    frequency = 500;
                    break;
                case 1:
                    frequency = 900;
                    break;
                case 2:
                    frequency = 1500;
                    break;
                case 3:
                    frequency = 2500;
                    break;
                case 4:
                    frequency = 4500;
                    break;
                default:
                    return;
            }

            // Initialise the sensor
            try
            {
                int syserrno = 0;
                int retval = this.sys.MFEndatInitSensor((ulong)this.channelComboBox.SelectedIndex,
                    (ulong)0,
                    frequency,
                    0, 0, 0, 0,
                    out syserrno);
                if (retval != 0)
                {
                    this.currentStatusLabel.Text = "MFEndatInitSensor error : " + retval.ToString() + ". syserrno : " + syserrno.ToString();
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }
            catch (Exception ex)
            {
                this.currentStatusLabel.Text = "MFEndatInitSensor error : " + ex.Message;
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            // Success
            this.currentStatusLabel.Text = "MFEndatInitSensor ok";
            this.currentStatusLabel.ForeColor = Color.Green;
        }

        /// <summary>
        /// Click on get position
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getPositionButton_Click(object sender, EventArgs e)
        {
            ulong positionlow = 0;
            ulong positionhigh = 0;

            // Clear items
            this.configurationListView.Items.Clear();

            this.currentStatusLabel.Text = "";
            this.currentStatusLabel.Invalidate();

            // Get properties
            try
            {
                MSXE173xWeb.DefaultResponse response = this.sys.MFEndatGetPosition((ulong)channelComboBox.SelectedIndex,
                    (ulong)0,
                    0, 0, 0, 0,
                    out positionlow, out positionhigh);
                if (response.iReturnValue != 0)
                {
                    this.currentStatusLabel.Text = "MFEndatGetPosition error : " + response.iReturnValue.ToString() + ". syserrno : " + response.syserrno.ToString();
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }
            catch (Exception ex)
            {
                this.currentStatusLabel.Text = "MFEndatGetPosition error : " + ex.Message;
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            // Success
            this.currentStatusLabel.Text = "MFEndatGetPosition ok";
            this.currentStatusLabel.ForeColor = Color.Green;

            // Add the elements to the list
            this.configurationListView.Items.Add(new ListViewItem(new string[2] { "Position (low bits)", positionlow.ToString() }));
            this.configurationListView.Items.Add(new ListViewItem(new string[2] { "Position (high bits)", positionhigh.ToString() }));
        }
    }
}