using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.NetworkInformation;

namespace GetSensorProperties
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Web reference to the system
        /// </summary>
        private MSXE173xWeb.MSXE17xx sys;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Occurs when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            this.ipTextBox.Text = Properties.Settings.Default.ip;

            // Set to default
            this.channelComboBox.SelectedIndex = 0;
            this.frequencyComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// User clicks on ok
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void okButton_Click(object sender, EventArgs e)
        {
            // Disable the button
            this.okButton.Enabled = false;
            this.ipTextBox.Enabled = false;

            // Start the worker to check the system connection using ping
            systemConnectionBackgroundWorker.RunWorkerAsync();
        }

        /// <summary>
        /// Job of the worker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void systemConnectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            Ping ping = new Ping();
            PingReply reply = ping.Send(this.ipTextBox.Text, 2000);
            if (reply.Status == IPStatus.Success)
            {
                worker.ReportProgress(100);
            }
            else
            {
                worker.ReportProgress(0);
            }
            return;
        }

        /// <summary>
        /// State has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void systemConnectionBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (e.ProgressPercentage == 100)
            {
                // success

                // Create the web reference
                this.sys = new MSXE173xWeb.MSXE17xx();
                this.sys.Url = "http://" + this.ipTextBox.Text + ":5555";
                this.sys.Timeout = 2000;

                // update status
                this.currentStatusLabel.Text = "Connected to " + this.ipTextBox.Text;
                this.currentStatusLabel.ForeColor = Color.Green;
                this.channelSelectionGroupBox.Enabled = true;
            }
            else
            {
                // connection problem
                this.currentStatusLabel.Text = "Cannot open connection to " + this.ipTextBox.Text;
                this.currentStatusLabel.ForeColor = Color.Red;
                this.ipTextBox.Enabled = true;
                this.okButton.Enabled = true;
            }
        }

        /// <summary>
        /// Click on the initialise button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initButton_Click(object sender, EventArgs e)
        {
            ulong frequency = 0;

            this.currentStatusLabel.Text = "";
            this.currentStatusLabel.Invalidate();

            // Get the frequency
            switch (this.frequencyComboBox.SelectedIndex)
            {
                case 0:
                    frequency = 500;
                    break;
                case 1:
                    frequency = 900;
                    break;
                case 2:
                    frequency = 1500;
                    break;
                case 3:
                    frequency = 2500;
                    break;
                case 4:
                    frequency = 4500;
                    break;
                default:
                    return;
            }

            // Initialise the sensor
            try
            {
                int syserrno = 0;
                int retval = this.sys.MFEndatInitSensor((ulong)this.channelComboBox.SelectedIndex,
                    (ulong)0,
                    frequency,
                    0, 0, 0, 0,
                    out syserrno);
                if ( retval != 0 )
                {
                    this.currentStatusLabel.Text = "MFEndatInitSensor error : " + retval.ToString() + ". syserrno : " + syserrno.ToString();
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }
            catch (Exception ex)
            {
                this.currentStatusLabel.Text = "MFEndatInitSensor error : " + ex.Message;
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            // Success
            this.currentStatusLabel.Text = "MFEndatInitSensor ok";
            this.currentStatusLabel.ForeColor = Color.Green;

            // Clear items
            this.configurationListView.Items.Clear();

            // Update
            this.configurationListView.Refresh();
        }

        /// <summary>
        /// Click on get configuration
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getConfigurationButton_Click(object sender, EventArgs e)
        {
            ulong idNumberLsb = 0;
            ulong idNumberMsb = 0;
            ulong serialNumberLsb = 0;
            ulong serialNumberMsb = 0;
            ulong model = 0;
            ulong mode = 0;
            ulong positionSize = 0;
            ulong signalPeriod = 0;
            ulong stepPerRevolution = 0;
            ulong numberOfRevolution = 0;
            ulong scalingFactor = 0;
            ulong additionalData = 0;

            // Clear items
            this.configurationListView.Items.Clear();

            this.currentStatusLabel.Text = "";
            this.currentStatusLabel.Invalidate();

            // Get properties
            try
            {
                MSXE173xWeb.DefaultResponse response = this.sys.MFEndatGetSensorProperties((ulong)channelComboBox.SelectedIndex,
                    (ulong)0,
                    0, 0, 0, 0,
                    out idNumberLsb,
                    out idNumberMsb,
                    out serialNumberLsb,
                    out serialNumberMsb,
                    out model,
                    out mode,
                    out positionSize,
                    out signalPeriod,
                    out stepPerRevolution,
                    out numberOfRevolution,
                    out scalingFactor,
                    out additionalData);
                if (response.iReturnValue != 0)
                {
                    this.currentStatusLabel.Text = "MFEndatGetSensorProperties error : " + response.iReturnValue.ToString() + ". syserrno : " + response.syserrno.ToString();
                    this.currentStatusLabel.ForeColor = Color.Red;
                    return;
                }
            }
            catch (Exception ex)
            {
                this.currentStatusLabel.Text = "MFEndatGetSensorProperties error : " + ex.Message;
                this.currentStatusLabel.ForeColor = Color.Red;
                return;
            }

            // Success
            this.currentStatusLabel.Text = "MFEndatGetSensorProperties ok";
            this.currentStatusLabel.ForeColor = Color.Green;

            // Add the elements to the list
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "idNumberLsb", idNumberLsb.ToString(), "ID - Low bits" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "idNumberMsb", idNumberMsb.ToString(), "ID - High bits" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "serialNumberLsb", serialNumberLsb.ToString(), "Serialnumber - Low bits" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "serialNumberMsb", serialNumberMsb.ToString(), "Serialnumber - High bits" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "model", model.ToString(), "Model of the sensor. (0-3) Incremental liner encoder (4 or 6) Absolute linear encoder (8-11) Incremental rotary encoder or angle encoder (12) Singleturn encoder (13 or 14) Multiturn encoder" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "mode", mode.ToString(), "1: EnDat 2.2 sensor" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "positionSize", positionSize.ToString(), "Size of the position value send by the sensor (in bits)" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "signalPeriod", signalPeriod.ToString(), "Signal period length or signal periods per revolution for incremental output signals" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "stepPerRevolution", stepPerRevolution.ToString(), "Measuring step length or measuring steps per revolution with serial data transfer" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "numberOfRevolution", numberOfRevolution.ToString(), "Distinguishable revolutions (only for multiturn encoders)" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "scalingFactor", scalingFactor.ToString(), "Scaling factor for resolution" }));
            this.configurationListView.Items.Add(new ListViewItem(new string[3] { "additionalData", additionalData.ToString(), "Supported additional data" }));

            // Update
            this.configurationListView.Refresh();
        }

        /// <summary>
        /// User changed the selected index
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void channelComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Clear items
            this.configurationListView.Items.Clear();

            // Update
            this.configurationListView.Refresh();
        }
    }
}